#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import division
import unittest
import os
import sys
import tempfile
import shutil
py_rumd_lib = {2:"../../../lib", 3:"../../../lib_py3"}[sys.version_info[0]]
sys.path.insert(1, py_rumd_lib)
sys.path.insert(1, "../../../Python")

import rumd
from rumd.Simulation import Simulation



xyz = """\
     3
ioformat=2 dt=0.005000000 sim_box=RectangularSimulationBox,10.0,10.0,10.0 numTypes=2 mass=1.000,1.000 columns=type,x,y,z
0     0.000 0.000 0.000
1     1.300 0.000 0.000
1     0.800 3.200 0.000
"""

class TestAdaptersRUMD(unittest.TestCase):

    def setUp(self):
        self.tdir = tempfile.mkdtemp()
        self.fout = self.tdir + '/test_adapter_rumd_out.xyz.gz'
        self.dout = self.tdir + '/test_adapter_rumd'
        self.finp = self.tdir + '/test_adapter_rumd_in.xyz'
        fh = open(self.finp, 'w')
        fh.write(xyz)
        fh.close()

        self.s = Simulation(self.finp, verbose=False)
        #self.s.sample.SetOutputDirectory(self.dout)
        #self.s.suppressAllOutput = True
        self.p_IPLn = rumd.Pot_IPL_n(1.0, cutoff_method = rumd.ShiftedForce)
        self.p_IPLn.SetVerbose(False)
        self.p_IPLn.SetParams(0, 0, 1., 1.0, 5.0)
        self.p_IPLn.SetParams(0, 1, 1., 0.5, 5.0)
        self.p_IPLn.SetParams(1, 1, 1., 0.25, 5.0)

        self.p_Coul = rumd.Pot_ShiftedForceCoulomb()
        self.p_Coul.SetVerbose(False)
        self.p_Coul.SetParams(0, 0, 1.0, 5.0)
        self.p_Coul.SetParams(0, 1, 0.5, 5.0)
        self.p_Coul.SetParams(1, 1, 0.25, 5.0)
    
    def test_pe(self):

        self.s.SetPotential(self.p_IPLn)
        self.s.sample.CalcF()
        pe_IPLn = self.s.sample.GetPotentialEnergy()

        self.s.SetPotential(self.p_Coul)
        self.s.sample.CalcF()
        pe_Coul = self.s.sample.GetPotentialEnergy()
        self.assertLess(abs((pe_IPLn-pe_Coul)/pe_Coul), 0.001)
        
        
    def tearDown(self):
        if os.path.exists(self.finp):
            os.remove(self.finp)
        if os.path.exists(self.fout):
            os.remove(self.fout)
        if os.path.exists(self.dout):
            shutil.rmtree(self.dout)
        if os.path.exists(self.tdir):
            shutil.rmtree(self.tdir)

if __name__ == '__main__':
    unittest.main()

