#!/usr/bin/env python
"""Checks that the function RunCompress works"""

from __future__ import absolute_import
from __future__ import print_function
import unittest
import sys
py_rumd_lib = {2:"../../../lib", 3:"../../../lib_py3"}[sys.version_info[0]]
sys.path.insert(1, py_rumd_lib)
sys.path.insert(1, "../../../Python")


import rumd
import rumd.Simulation
from rumd.RunCompress import RunCompress
# it is essential that there is no doubt about which version is actually run!
print("File used to import the rumd module", rumd.__file__)

class TestRunCompress(unittest.TestCase):
    """Unit-testing class for testing RunCompress """

    def test_compress(self):
        """ set up a simulation and use RunCompress function to change
        density"""
        sim = rumd.Simulation.Simulation("dimer128_rho0.85_T0.5.xyz.gz")
        # read topology file
        sim.ReadMoleculeData("dimer128_rho0.85_T0.5.top")

        # create integrator object
        itg = rumd.IntegratorNVT(timeStep=0.0025, targetTemperature=1.0)
        sim.SetIntegrator(itg)

        # create pair potential object
        potential = rumd.Pot_LJ_12_6(cutoff_method=rumd.ShiftedPotential)
        potential.SetParams(i=0, j=0, Epsilon=1.0, Sigma=1.0, Rcut=2.5)
        sim.SetPotential(potential)

        # define harmonic bond and its parameters for bonds of type 0
        sim.SetBondHarmonic(bond_type=0, lbond=1.0, ks=200.0)

        # first make a very low denstiy system
        RunCompress(sim, 0.001, verbose=False, num_scale_steps=10, niter_per_scale=50)
        # Now re-compress
        RunCompress(sim, 0.9, verbose=False, num_scale_steps=100, niter_per_scale=100)
        # need to something to check quantitatively. Need to allow a large
        # tolerance though
        sim.sample.CalcF()
        reference_energy = -1170.
        tolerance = 100.
        print("pe - pe_ref", sim.sample.GetPotentialEnergy() - reference_energy)
        self.assertLess(abs(sim.sample.GetPotentialEnergy() - reference_energy), tolerance)

if __name__ == '__main__':
    unittest.main()
