from __future__ import absolute_import
from __future__ import print_function
import sys
py_rumd_lib = {2:"../../../lib", 3:"../../../lib_py3"}[sys.version_info[0]]
sys.path.insert(1, py_rumd_lib)
sys.path.insert(1, "../../../Python")

import rumd
# it is essential that there is no doubt about which version is actually run!
print("File used to import the rumd module",rumd.__file__)

from rumd.Simulation import Simulation
from rumd.RUMD_Test import RUMD_Test

class Test_Compare_NB_Methods (RUMD_Test):
    """
    Binary LJ system with NVE dynamics, run with N2 method as "reference", and
    then with sort method, compare as usual.
    """
    def RunSimulation(self, mode, numParticles, pb, tp):

        # set the number of time-steps depending on the mode (assigned
        # as a class attribute so the calling function can access it later)
        self.numTimeSteps = 50
                
        # sample and simulation
        startConfg = "startN%d.xyz.gz" % numParticles

        
        sim = Simulation(startConfg, pb, tp, verbose=False)


        sim.SetOutputMetaData("energies",thermostat_Ps=True)

        # integrator
        itg = rumd.IntegratorNVE(timeStep=0.005)

        # Create LJ pair-potential
        potential = rumd.Pot_LJ_12_6(cutoff_method=rumd.ShiftedForce)
        # Set up parameters for Lennard-Jones interactions ( Kob & Andersen )
        # i  j   Sig   Eps  Rcut (in units of Sigma_ij)
        potential.SetParams(0, 0, 1.00, 1.00, 1.5)
        potential.SetParams(0, 1, 0.80, 1.50, 1.5)
        potential.SetParams(1, 1, 0.88, 0.50, 1.5)
        
        sim.SetIntegrator(itg)
        sim.SetPotential(potential)
        
        potential.SetNbListSkin(0.1)
        potential.SetNB_Method("n2")

        # tune output depending on the mode
        if mode != "short":
            raise ValueError("This test is only available in \"short\" mode")
        sim.SetOutputScheduling("energies","linear",interval=1)
        sim.SetOutputScheduling("trajectory","linear",interval=10)
        sim.SetBlockSize(50)

        sim.SetOutputDirectory(self.ref_directory)
        
        sim.Run(self.numTimeSteps)

        sim.sample.ReadConf(startConfg)
        potential.SetNB_Method("sort")

        sim.SetOutputDirectory(self.out_directory)
        sim.Run(self.numTimeSteps)



if __name__ == "__main__":

    rumd_test = Test_Compare_NB_Methods()

    # standard short test to be called from makefile
    # For generating reference data and running stress and performance modes
    # there are corresponding scripts to be submitted to pbs

    if "gen_ref" in sys.argv:
        raise ValueError("This test does not use reference data")

    sys.exit(rumd_test.RunShort(False) )
    
    # for individual runs, could do for example
    #sys.exit(rumd_test.Test("short", 1000, 16, 8))
