#!/usr/bin/env python

#PBS -N test_reduction
#PBS -o log.o
#PBS -e log.e


from __future__ import absolute_import
from __future__ import division
import os
import sys
import unittest

if "PBS_O_WORKDIR" in os.environ:
    workingDir = os.environ["PBS_O_WORKDIR"]
    os.chdir(workingDir)
    sys.path.append(".")




sys.path.insert(1, "../../../lib")
sys.path.insert(1, "../../../Python")

import rumd
from rumd.Simulation import Simulation


class TestReductionBug(unittest.TestCase):
    def setUp(self):
        self.sim = Simulation("startN131072.xyz.gz")
        self.itg = rumd.IntegratorNVT(timeStep=0.0025, targetTemperature=0.7)
        self.sim.SetIntegrator(self.itg)
        self.pot = rumd.Pot_LJ_12_6(cutoff_method=rumd.ShiftedPotential)
        self.pot.SetParams(i=0, j=0, Sigma=1.0, Epsilon=1.0, Rcut=2.5)
        self.sim.SetPotential(self.pot)
        self.count = 0

    def CheckThermostatKE(self):
        ke = self.itg.GetKineticEnergy()
        ke_th = self.itg.GetThermostatKineticEnergy()
        if ke_th > 0. and ke_th / ke > 1.002:
            raise RuntimeError("th_KE exceeds KE by factor %.5f at step %d; " \
                               " values %.6f, %.6f" % (ke_th/ke,
                                                       self.count, ke, ke_th))
        self.count += 1

    def test_reduction(self):

        self.itg.SetCopyThermostatKineticEnergy(True)
        self.sim.SetOutputScheduling("energies", "linear", interval=1)
        self.sim.SetOutputMetaData("energies", thermostat_KE=True)
        self.sim.SetOutputScheduling("trajectory", "none")

        blk_size = 256
        num_blocks = 400
        self.sim.SetBlockSize(blk_size)

        self.sim.SetRuntimeAction("check_thermostatKE",
                                  self.CheckThermostatKE, interval=1)
        self.sim.Run(blk_size * num_blocks)

if __name__ == '__main__':
    unittest.main()
